@@ -1,6 +1,7 @@
 <![CDATA[#include <libgda/libgda.h>
+#include <sql-parser/gda-sql-parser.h>
 
-GdaConnection *open_connection (GdaClient *client);
+GdaConnection *open_connection (void);
 void display_products_contents (GdaConnection *cnc);
 void create_table (GdaConnection *cnc);
 void run_sql_non_select (GdaConnection *cnc, const gchar *sql);
@@ -10,14 +11,10 @@ main (int argc, char *argv[])
 {
         gda_init ("SimpleExample", "1.0", argc, argv);
 
-        GdaClient *client;
         GdaConnection *cnc;
 
-        /* Create a GdaClient object which is the central object which manages all connections */
-        client = gda_client_new ();
-
 	/* open connections */
-	cnc = open_connection (client);
+	cnc = open_connection ();
 	create_table (cnc);
 	display_products_contents (cnc);
         gda_connection_close (cnc);
@@ -29,21 +26,28 @@ main (int argc, char *argv[])
  * Open a connection to the example.db file
  */
 GdaConnection *
-open_connection (GdaClient *client)
+open_connection (void)
 {
         GdaConnection *cnc;
         GError *error = NULL;
-        cnc = gda_client_open_connection_from_string (client, "SQLite", 
-						      "DB_DIR=.;DB_NAME=example_db", 
-						      NULL, NULL,
-						      GDA_CONNECTION_OPTIONS_DONT_SHARE,
-						      &error);
+	GdaSqlParser *parser;
+
+	/* open connection */
+        cnc = gda_connection_open_from_string ("SQLite", "DB_DIR=.;DB_NAME=example_db", NULL,
+					       GDA_CONNECTION_OPTIONS_NONE, &error);
         if (!cnc) {
-                g_print ("Could not open connection to SQLite database in "
-			 "example_db.db file: %s\n",
+                g_print ("Could not open connection to SQLite database in example_db.db file: %s\n",
                          error && error->message ? error->message : "No detail");
                 exit (1);
         }
+
+	/* create an SQL parser */
+	parser = gda_connection_create_parser (cnc);
+	if (!parser) /* @cnc doe snot provide its own parser => use default one */
+		parser = gda_sql_parser_new ();
+	/* attach the parser object to the connection */
+	g_object_set_data_full (G_OBJECT (cnc), "parser", parser, g_object_unref);
+
         return cnc;
 }
 
@@ -54,8 +58,7 @@ void
 create_table (GdaConnection *cnc)
 {
 	run_sql_non_select (cnc, "DROP table IF EXISTS products");
-        run_sql_non_select (cnc, "CREATE table products "
-			    "(ref string not null primary key, "
+        run_sql_non_select (cnc, "CREATE table products (ref string not null primary key, "
                             "name string not null, price real)");
 	run_sql_non_select (cnc, "INSERT INTO products VALUES ('p1', 'chair', 2.0)");
 	run_sql_non_select (cnc, "INSERT INTO products VALUES ('p2', 'table', 5.0)");
@@ -70,14 +73,15 @@ void
 display_products_contents (GdaConnection *cnc)
 {
 	GdaDataModel *data_model;
-	GdaCommand *command;
+	GdaSqlParser *parser;
+	GdaStatement *stmt;
 	gchar *sql = "SELECT ref, name, price FROM products";
 	GError *error = NULL;
 
-	command = gda_command_new (sql, GDA_COMMAND_TYPE_SQL, 
-				   GDA_COMMAND_OPTION_STOP_ON_ERRORS);
-	data_model = gda_connection_execute_select_command (cnc, command, NULL, &error);
-	gda_command_free (command);
+	parser = g_object_get_data (G_OBJECT (cnc), "parser");
+	stmt = gda_sql_parser_parse_string (parser, sql, NULL, NULL);
+	data_model = gda_connection_statement_execute_select (cnc, stmt, NULL, &error);
+	g_object_unref (stmt);
         if (!data_model) 
                 g_error ("Could not get the contents of the 'products' table: %s\n",
                          error && error->message ? error->message : "No detail");
@@ -91,15 +95,20 @@ display_products_contents (GdaConnection *cnc)
 void
 run_sql_non_select (GdaConnection *cnc, const gchar *sql)
 {
-        GdaCommand *command;
+        GdaStatement *stmt;
         GError *error = NULL;
         gint nrows;
+	const gchar *remain;
+	GdaSqlParser *parser;
 
-        command = gda_command_new (sql, GDA_COMMAND_TYPE_SQL, 0);
-        nrows = gda_connection_execute_non_select_command (cnc, command, NULL, &error);
-        gda_command_free (command);
+	parser = g_object_get_data (G_OBJECT (cnc), "parser");
+	stmt = gda_sql_parser_parse_string (parser, sql, &remain, &error);
+	if (remain) 
+		g_print ("REMAINS: %s\n", remain);
+
+        nrows = gda_connection_statement_execute_non_select (cnc, stmt, NULL, NULL, &error);
         if (nrows == -1)
-                g_error ("NON SELECT error: %s\n", 
-			 error && error->message ? error->message : "no detail");
+                g_error ("NON SELECT error: %s\n", error && error->message ? error->message : "no detail");
+	g_object_unref (stmt);
 }
 ]]>
