@@ -1,6 +1,6 @@
 /* gda-data-proxy.h
  *
- * Copyright (C) 2005 - 2007 Vivien Malerba
+ * Copyright (C) 2005 - 2008 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -23,7 +23,7 @@
 #define __GDA_DATA_PROXY_H_
 
 #include "gda-decl.h"
-#include <libgda/gda-object.h>
+#include <glib-object.h>
 #include <libgda/gda-value.h>
 #include <libgda/gda-enums.h>
 
@@ -41,13 +41,16 @@ extern GQuark gda_data_proxy_error_quark (void);
 
 typedef enum {
 	GDA_DATA_PROXY_COMMIT_ERROR,
-	GDA_DATA_PROXY_COMMIT_CANCELLED
+	GDA_DATA_PROXY_COMMIT_CANCELLED,
+	GDA_DATA_PROXY_READ_ONLY_VALUE,
+	GDA_DATA_PROXY_READ_ONLY_ROW,
+	GDA_DATA_PROXY_FILTER_ERROR
 } GdaDataProxyError;
 
 /* struct for the object's data */
 struct _GdaDataProxy
 {
-	GdaObject               object;
+	GObject                 object;
 	GdaDataProxyPrivate    *priv;
 };
 
@@ -55,7 +58,7 @@ struct _GdaDataProxy
 /* struct for the object's class */
 struct _GdaDataProxyClass
 {
-	GdaObjectClass          parent_class;
+	GObjectClass            parent_class;
 
 	void                 (* row_delete_changed)   (GdaDataProxy *proxy, gint row, gboolean to_be_deleted);
 
@@ -103,6 +106,10 @@ void              gda_data_proxy_set_sample_start         (GdaDataProxy *proxy,
 gint              gda_data_proxy_get_sample_start         (GdaDataProxy *proxy);
 gint              gda_data_proxy_get_sample_end           (GdaDataProxy *proxy);
 
+gboolean          gda_data_proxy_set_filter_expr          (GdaDataProxy *proxy, const gchar *filter_expr, GError **error);
+const gchar      *gda_data_proxy_get_filter_expr          (GdaDataProxy *proxy);
+gint              gda_data_proxy_get_filtered_n_rows      (GdaDataProxy *proxy);
+
 G_END_DECLS
 
 #endif
