@@ -1,6 +1,6 @@
 /* gda-handler-time.c
  *
- * Copyright (C) 2003 - 2007 Vivien Malerba
+ * Copyright (C) 2003 - 2008 Vivien Malerba
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -103,7 +103,7 @@ gda_handler_time_get_type (void)
 			NULL
 		};
 
-		type = g_type_register_static (GDA_TYPE_OBJECT, "GdaHandlerTime", &info, 0);
+		type = g_type_register_static (G_TYPE_OBJECT, "GdaHandlerTime", &info, 0);
 		g_type_add_interface_static (type, GDA_TYPE_DATA_HANDLER, &data_entry_info);
 	}
 	return type;
@@ -163,8 +163,8 @@ gda_handler_time_init (GdaHandlerTime *hdl)
 	hdl->priv->str_locale->current_offset = 0;
 	hdl->priv->str_locale->separator = '-';
 
-	gda_object_set_name (GDA_OBJECT (hdl), _("InternalTime"));
-	gda_object_set_description (GDA_OBJECT (hdl), _("Time, Date and TimeStamp representation"));
+	g_object_set_data (G_OBJECT (hdl), "name", _("InternalTime"));
+	g_object_set_data (G_OBJECT (hdl), "descr", _("Time, Date and TimeStamp representation"));
 }
 
 static void
@@ -178,8 +178,6 @@ gda_handler_time_dispose (GObject *object)
 	hdl = GDA_HANDLER_TIME (object);
 
 	if (hdl->priv) {
-		gda_object_destroy_check (GDA_OBJECT (object));
-
 		g_free (hdl->priv->valid_g_types);
 		hdl->priv->valid_g_types = NULL;
 
@@ -206,7 +204,7 @@ gda_handler_time_new (void)
 {
 	GObject *obj;
 
-	obj = g_object_new (GDA_TYPE_HANDLER_TIME, "dict", NULL, NULL);
+	obj = g_object_new (GDA_TYPE_HANDLER_TIME, NULL);
 	handler_compute_locale (GDA_HANDLER_TIME (obj));
 
 	return (GdaDataHandler *) obj;
@@ -1112,5 +1110,5 @@ gda_handler_time_get_descr (GdaDataHandler *iface)
 	hdl = GDA_HANDLER_TIME (iface);
 	g_return_val_if_fail (hdl->priv, NULL);
 
-	return gda_object_get_description (GDA_OBJECT (hdl));
+	return g_object_get_data (G_OBJECT (hdl), "descr");
 }
